
/****************************************************************************
 ** Licensed Materials - Property of IBM 
 ** IBM InfoSphere Change Data Capture
 ** 5724-U70
 ** 
 ** (c) Copyright IBM Corp. 2014 All rights reserved.
 ** 
 ** The following sample of source code ("Sample") is owned by International 
 ** Business Machines Corporation or one of its subsidiaries ("IBM") and is 
 ** copyrighted and licensed, not sold. You may use, copy, modify, and 
 ** distribute the Sample in any form without payment to IBM.
 ** 
 ** The Sample code is provided to you on an "AS IS" basis, without warranty of 
 ** any kind. IBM HEREBY EXPRESSLY DISCLAIMS ALL WARRANTIES, EITHER EXPRESS OR Mon4
 ** IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
 ** MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. Some jurisdictions do 
 ** not allow for the exclusion or limitation of implied warranties, so the above 
 ** limitations or exclusions may not apply to you. IBM shall not be liable for 
 ** any damages you suffer as a result of using, copying, modifying or 
 ** distributing the Sample, even if IBM has been advised of the possibility of 
 ** such damages.
 *****************************************************************************/

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Properties;

import com.ibm.replication.cdc.scripting.EmbeddedScript;
import com.ibm.replication.cdc.scripting.EmbeddedScriptException;
import com.ibm.replication.cdc.scripting.Result;
import com.ibm.replication.cdc.scripting.ResultStringTable;

public class Mon4Dashboard {

	static final String propertiesFile = "Mon4Dashboard.properties";
	static String mode = "true"; //"true" is real, "false" is test mode
	static String logPrintFlag = "false";   //"true" is log print, "false" is log don't print

	public static void main(String[] args) {

		EmbeddedScript script = new EmbeddedScript();
		String ds = "";

		/********************* Insert DB Daou Kwon-Hyeak-Se ************************/
		Connection con = null;
		Statement stmt = null;
		ResultSet rs = null;
		PreparedStatement dml_stmt = null;
		/********************* Insert DB Daou Kwon-Hyeak-Se ************************/

		try
		{
			Properties properties = new Properties(); 
			properties.load(new FileInputStream(propertiesFile));
			String hostname = properties.getProperty("hostname");
			String portname = properties.getProperty("portname");
			String username = properties.getProperty("username");
			String password = properties.getProperty("password");
			String sourceAgent  = properties.getProperty("sourceDatastore");
			String targetAgent  = properties.getProperty("targetDatastore");
			String subscriptionNames = properties.getProperty("subscriptionNames");
			String subscriptionName[] = subscriptionNames.split(",");
			int numberOfSubscriptions = subscriptionName.length;

			/********************* Insert DB  ************************/
			String url = properties.getProperty("url");
			String dbUser = properties.getProperty("dbId");
			String dbPassword = properties.getProperty("dbPassword");
			String driverName = properties.getProperty("jdbcDriver");
			String schemaName = properties.getProperty("schemaName");
			String dataStatusTable = properties.getProperty("dataStatusTable");
			String serverStatusTable = properties.getProperty("serverStatusTable");
			if(properties.getProperty("mode")!=null){
				mode = properties.getProperty("mode");
			}
			if(properties.getProperty("logPrintFlag")!=null){
				logPrintFlag = properties.getProperty("logPrintFlag");
			}		   
	
			/********************* Insert DB  ************************/

			try
			{
				/********************* Insert DB  Start ************************/
				try {
					Class.forName(driverName).newInstance();
					logPrint("Driver Loading");
					System.out.println("url : " + url);
					System.out.println("dbUser : " + dbUser);
					System.out.println("dbPassword : " + dbPassword);
					con = DriverManager.getConnection(url, dbUser, dbPassword);
					logPrint("get Connection Object");
				} catch (Exception e) {
					// TODO: handle exception
					e.printStackTrace();
				}
				/********************* Insert DB End ************************/

				// Connect to Access Server
				script.open();
				ds = "AS";
				script.execute("CONNECT SERVER HOSTNAME " + hostname + " PORT " + portname + " USERNAME " + username + " PASSWORD " + password);
				ds = sourceAgent;
				
				// Make source datastore connection string. Don't add "context sourece" if source and target agent name is same.
				String datastoreConnect = "connect datastore name " + sourceAgent;
				if (!sourceAgent.equals(targetAgent)){ 
					datastoreConnect = datastoreConnect + " context source";
				}
				script.execute(datastoreConnect);
				ds = "OK";
				Result result = script.getResult();
				try {
					String sql = "INSERT INTO "+schemaName+"."+serverStatusTable+" VALUES (CURRENT TIMESTAMP, ?, ?)";
					System.out.println("schemaName : "  + schemaName);
					System.out.println("serverStatusTable : "  + serverStatusTable);
					dml_stmt = con.prepareStatement(sql);
					dml_stmt.setString(1, sourceAgent);
					dml_stmt.setString(2, "1");						
					logPrint("commit");
					if(dml_stmt.executeUpdate()==PreparedStatement.RETURN_GENERATED_KEYS){
						logPrint("CDC_SERVER_STATUS insert success");
					}else{
						logPrint("CDC_SERVER_STATUS insert fail");
					}							
					con.commit();
					logPrint("commit");

				} catch (SQLException e2) {
					// TODO: handle exception
					e2.printStackTrace();
				}
				
				// Connect to target agent if source and target agent is different
				if (!sourceAgent.equals(targetAgent)){
					ds = targetAgent;
					script.execute("connect datastore name " + targetAgent + " context target");
					try {
						String sql = "INSERT INTO "+schemaName+"."+serverStatusTable+" VALUES (CURRENT TIMESTAMP, ?, ?)";
						System.out.println("schemaName : "  + schemaName);
						System.out.println("serverStatusTable : "  + serverStatusTable);
						dml_stmt = con.prepareStatement(sql);
						dml_stmt.setString(1, targetAgent);
						dml_stmt.setString(2, "1");						
						logPrint("commit");
						if(dml_stmt.executeUpdate()==PreparedStatement.RETURN_GENERATED_KEYS){
							logPrint("CDC_SERVER_STATUS insert success");
						}else{
							logPrint("CDC_SERVER_STATUS insert fail");
						}							
						con.commit();
						logPrint("commit");

					} catch (SQLException e2) {
						// TODO: handle exception
						e2.printStackTrace();
					}
					ds = "OK";
				}

				for (int k = 0 ; k < numberOfSubscriptions ; k++){
					if (subscriptionName[k].startsWith(" ")) {
						subscriptionName[k] = subscriptionName[k].substring(1);
					}

					script.execute("monitor replication filter subscription name " + subscriptionName[k]);
					result = script.getResult();
					ResultStringTable resultTable = (ResultStringTable) result;
					String status = resultTable.getValueAt(0,1);
					logPrint("Status : " + status);

					/********************* Insert DB  Start ************************/
				
					String statusFlag = "1";
					String ti = "";
					String tu = "";
					String td = "";
					String s = "";
					/********************* Insert DB End ************************/

					if (status.equals("Mirror Continuous") || status.equals(" ̷")) {
						script.execute("monitor subscription activity name " + subscriptionName[k]);
						result = script.getResult();
						resultTable = (ResultStringTable) result;
						//					   int si = Integer.parseInt(resultTable.getValueAt(6,1).replaceAll(",",""));
						//					   int su = Integer.parseInt(resultTable.getValueAt(7,1).replaceAll(",",""));
						//					   int sd = Integer.parseInt(resultTable.getValueAt(8,1).replaceAll(",",""));

						//					   int ti = Integer.parseInt(resultTable.getValueAt(21,1).replaceAll(",",""));
						//					   int tu = Integer.parseInt(resultTable.getValueAt(22,1).replaceAll(",",""));
						//					   int td = Integer.parseInt(resultTable.getValueAt(23,1).replaceAll(",",""));
						ti = resultTable.getValueAt(21,1).replaceAll(",","");
						tu = resultTable.getValueAt(22,1).replaceAll(",","");
						td = resultTable.getValueAt(23,1).replaceAll(",","");

						//					   logPrint("Source Insert : " + si);
						//					   logPrint("Source Update : " + su);
						//					   logPrint("Source Delete : " + sd);
						logPrint("Target Insert : " + ti);
						logPrint("Target Update : " + tu);
						logPrint("Target Delete : " + td);

						script.execute("monitor subscription latency name " + subscriptionName[k]);
						result = script.getResult();
						resultTable = (ResultStringTable) result;
						//					   int s = Integer.parseInt(resultTable.getValueAt(0,2).replaceAll(",",""));
						s = resultTable.getValueAt(0,2).replaceAll(",","");
						//					   logPrint("Latency       : " + s);
					}else{//fail
						/********************* Insert DB  Start ************************/
						statusFlag = "0";
						/********************* Insert DB End ************************/
					}

					/********************* Insert DB Start ************************/


					String sql = "INSERT INTO "+schemaName+"."+dataStatusTable+" VALUES (CURRENT TIMESTAMP, ?, ?, ?, ?, ?, ?, ?, ?)";
					dml_stmt = con.prepareStatement(sql);
					if(statusFlag.equals("1")){
						dml_stmt.setString(1, subscriptionName[k]);
						dml_stmt.setString(2, sourceAgent);
						dml_stmt.setString(3, targetAgent);
						dml_stmt.setString(4, statusFlag);
						dml_stmt.setString(5, s);
						dml_stmt.setString(6, ti);
						dml_stmt.setString(7, tu);
						dml_stmt.setString(8, td);
					}else{
						dml_stmt.setString(1, subscriptionName[k]);
						dml_stmt.setString(2, sourceAgent);
						dml_stmt.setString(3, targetAgent);
						dml_stmt.setString(4, statusFlag);
						dml_stmt.setString(5, "0");
						dml_stmt.setString(6, "0");
						dml_stmt.setString(7, "0");
						dml_stmt.setString(8, "0");
					}

					if(dml_stmt.executeUpdate()==PreparedStatement.RETURN_GENERATED_KEYS){
						logPrint("insert success");
					}else{
						logPrint("insert fail");
					}							   
					/********************* Insert DB  End ************************/
				}
				/********************* Insert DB Start ************************/
				if(mode.equals("true")){ //real mode
					con.commit();
					logPrint("commit");
					dml_stmt.close();
					con.close();
				}else{		//test mode : print table datas
					con.commit();
					logPrint("commit");
					stmt = con.createStatement();
					rs = stmt.executeQuery("select count(*) from "+schemaName+"."+dataStatusTable);
					int rowCount = 0;
					while(rs.next()){
						rowCount = rs.getInt(1);
					}
					String sql = "select * from (select * from (select * from "+schemaName+"."+dataStatusTable + 
							" order by CheckTime asc fetch first "+rowCount+" rows only) as I order by CheckTime desc"+
							" fetch first " + numberOfSubscriptions + " rows only) as II order by CheckTime asc";
					rs = stmt.executeQuery(sql);
					//					rs = stmt.executeQuery("select * from "+schemaName+"."+dataStatusTable);
					//					rs = stmt.executeQuery("select TO_CHAR(CheckTime, 'YYYY-MM-DD HH:mm:ss') from "+schemaName+"."+dataStatusTable);
					logPrint("get ResultSet Object");
					logPrint(" 	CheckTime		Group		SourceDB	TargetDB	Status		Latency		Insert		Update		Delete");
					while(rs.next()){							
						String print = rs.getString(1) + "	" + rs.getString(2)+ "		" + rs.getString(3)
								+ "		" + rs.getString(4)+ "		" + rs.getString(5)+ "		" + rs.getString(6)
								+ "		" + rs.getString(7)+ "		" + rs.getString(8)+ "		" + rs.getString(9);
						//						String print = rs.getString(1);
						logPrint(print);
					}
					dml_stmt.close();		
					rs.close();
					stmt.close();
					con.close();
				}	
				/********************* Insert DB  End ************************/
			}
			catch (EmbeddedScriptException e)
			{
				// If fail to connect to Access Server or Agent, 
				int resultCode = e.getResultCode();
				String message = e.getResultMessage();
				logPrint(ds);
				String dataStore = "";
				if (ds.equals("AS")) {
					logPrint("Access Server Error");
					dataStore = "AS";
				}
				else if (ds.equals(sourceAgent)){
					logPrint("Source Error");
					dataStore = sourceAgent;
				}
				else if (ds.equals(targetAgent)){
					logPrint("Target Error");
					dataStore = targetAgent;
				}
				else {
					dataStore = "OTHER";
				}
				logPrint(resultCode + message);

				try {
					String sql = "INSERT INTO "+schemaName+"."+serverStatusTable+" VALUES (CURRENT TIMESTAMP, ?, ?)";
					System.out.println("schemaName : "  + schemaName);
					System.out.println("serverStatusTable : "  + serverStatusTable);
					dml_stmt = con.prepareStatement(sql);
					dml_stmt.setString(1, dataStore);
					dml_stmt.setString(2, "0");						
					logPrint("commit");
					if(dml_stmt.executeUpdate()==PreparedStatement.RETURN_GENERATED_KEYS){
						logPrint("CDC_SERVER_STATUS insert success");
					}else{
						logPrint("CDC_SERVER_STATUS insert fail");
					}							
					con.commit();
					logPrint("commit");
					dml_stmt.close();
					con.close();

				} catch (SQLException e2) {
					// TODO: handle exception
					e2.printStackTrace();
				}


			}catch (SQLException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}	 
		}
		catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} 

		catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		finally
		{
			script.close();
		}

	}	

	public static void logPrint(String log){
		if(logPrintFlag.equals("true")){
			System.out.println(log);
		}
	}
}